import EventEmitter from 'eventemitter3';
import type { ObjectGeneratorOptions } from './object-generator';
import Events from './element-events';
import type Document from './document';
import type { TypeCastTypes } from 'hadron-type-checker';
import type { BSONArray, BSONObject, BSONValue } from './utils';
import { ElementEvents } from '.';
export declare const DATE_FORMAT = "YYYY-MM-DD HH:mm:ss.SSS";
export { Events };
export declare function isInternalFieldPath(path: string | number): boolean;
export declare class Element extends EventEmitter {
    uuid: string;
    key: string | number;
    currentKey: string | number;
    value: BSONValue;
    currentValue: BSONValue;
    added: boolean;
    removed: boolean;
    elements?: LinkedList | null;
    originalExpandableValue?: BSONObject | BSONArray;
    parent: Element | Document | null;
    previousElement: Element | null;
    nextElement: Element | null;
    type: TypeCastTypes;
    currentType: TypeCastTypes;
    level: number;
    currentTypeValid?: boolean;
    invalidTypeMessage?: string;
    decrypted: boolean;
    bulkEdit(value: string): void;
    cancel(): void;
    constructor(key: string | number, value: BSONValue, added?: boolean, parent?: Element | Document | null, previousElement?: Element | null, nextElement?: Element | null);
    _getLevel(): number;
    edit(value: BSONValue): void;
    changeType(newType: TypeCastTypes): void;
    getRoot(): Document;
    get(key: string | number): Element | undefined;
    at(i: number): Element | undefined;
    next(): void;
    rename(key: string | number): void;
    generateObject(options?: ObjectGeneratorOptions): BSONValue;
    generateOriginalObject(options?: ObjectGeneratorOptions): BSONValue;
    insertAfter(element: Element, key: string | number, value: BSONValue): Element;
    insertEnd(key: string | number, value: BSONValue): Element;
    insertPlaceholder(): Element;
    insertSiblingPlaceholder(): Element;
    isAdded(): boolean;
    isBlank(): boolean;
    isCurrentTypeValid(): boolean;
    setValid(): void;
    setInvalid(value: BSONValue, newType: TypeCastTypes, message: string): void;
    isDuplicateKey(value: string | number): boolean;
    isEdited(): boolean;
    _valuesEqual(): boolean;
    _isObjectIdEqual(): boolean;
    isLast(): boolean;
    isRenamed(): boolean;
    isRevertable(): boolean;
    isRemovable(): boolean;
    isNotActionable(): boolean;
    isValueDecrypted(): boolean;
    containsDecryptedChildren(): boolean;
    isValueEditable(): boolean;
    isParentEditable(): boolean;
    _isKeyLegallyEditable(): boolean;
    isKeyEditable(): boolean;
    isInternalField(): boolean;
    isModified(): boolean;
    isRemoved(): boolean;
    isRoot(): false;
    remove(): void;
    revert(): void;
    _bubbleUp(evt: typeof Events[keyof typeof Events], ...data: BSONArray): void;
    _convertToEmptyObject(): void;
    _convertToEmptyArray(): void;
    _isElementEmpty(this: unknown, element: Element | undefined | null): boolean;
    _isExpandable(this: unknown, value: BSONValue): value is BSONObject | BSONArray;
    _generateElements(object: BSONObject | BSONArray): LinkedList;
    _key(key: string, index: number): string | number;
    _next(): void;
    _removeAddedElements(): void;
    static get Events(): typeof ElementEvents;
}
export declare class LinkedList {
    firstElement: Element | null;
    lastElement: Element | null;
    doc: Element | Document;
    originalDoc: BSONObject | BSONArray;
    keys: string[] | number[];
    size: number;
    loaded: number;
    _map: Record<string, Element>;
    at(index: number): Element | undefined;
    get(key: string | number): Element | undefined;
    constructor(doc: Document | Element, originalDoc: BSONObject | BSONArray);
    insertAfter(element: Element, key: string | number, value: BSONValue, added: boolean, parent: Element | Document): Element;
    updateKeys(element: Element, add: number): void;
    handleEmptyKeys(element: Element): void;
    insertBefore(element: Element, key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    insertBeginning(key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    insertEnd(key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    flush(): void;
    [Symbol.iterator](): Iterator<Element>;
    _needsLazyLoad(index: number): boolean;
    _needsStandardIteration(index: number): boolean;
    _lazyInsertEnd(key: string | number): Element;
    _insertEnd(key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    _insertBefore(element: Element, key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    _insertBeginning(key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    _insertAfter(element: Element, key: string | number, value: BSONValue, added: boolean, parent: Document | Element): Element;
    remove(element: Element): this;
    findIndex(el: Element): number;
}
export default Element;
//# sourceMappingURL=element.d.ts.map