"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectGenerator = void 0;
const DECRYPTED_KEYS = Symbol.for('@@mdb.decryptedKeys');
function maybeDecorateWithDecryptedKeys(object, element) {
    if (element.isValueDecrypted()) {
        if (!object[DECRYPTED_KEYS]) {
            Object.defineProperty(object, DECRYPTED_KEYS, {
                value: [],
                writable: true,
                configurable: true,
                enumerable: false,
            });
        }
        object[DECRYPTED_KEYS].push(String(element.currentKey));
    }
}
class ObjectGenerator {
    static generate(elements, options = {}) {
        if (elements) {
            const object = {};
            for (const element of elements) {
                if (options.excludeInternalFields && element.isInternalField()) {
                    continue;
                }
                if (!element.isRemoved() && element.currentKey !== '') {
                    object[element.currentKey] = element.generateObject(options);
                    maybeDecorateWithDecryptedKeys(object, element);
                }
            }
            return object;
        }
        return elements;
    }
    static generateOriginal(elements, options = {}) {
        if (elements) {
            const object = {};
            for (const element of elements) {
                if (options.excludeInternalFields && element.isInternalField()) {
                    continue;
                }
                if (!element.isAdded()) {
                    object[element.key] = element.generateOriginalObject(options);
                    maybeDecorateWithDecryptedKeys(object, element);
                }
            }
            return object;
        }
        return elements;
    }
    static generateArray(elements, options = {}) {
        if (elements) {
            const array = [];
            for (const element of elements) {
                if (options.excludeInternalFields && element.isInternalField()) {
                    continue;
                }
                if (!element.isRemoved()) {
                    if (element.elements) {
                        array.push(element.generateObject(options));
                    }
                    else {
                        array.push(element.currentValue);
                    }
                    maybeDecorateWithDecryptedKeys(array, element);
                }
            }
            return array;
        }
        return elements;
    }
    static generateOriginalArray(elements, options = {}) {
        if (elements) {
            const array = [];
            for (const element of elements) {
                if (options.excludeInternalFields && element.isInternalField()) {
                    continue;
                }
                if (element.originalExpandableValue) {
                    array.push(element.generateOriginalObject(options));
                }
                else if (!element.isAdded()) {
                    array.push(element.value);
                }
                maybeDecorateWithDecryptedKeys(array, element);
            }
            return array;
        }
        return elements;
    }
}
exports.ObjectGenerator = ObjectGenerator;
exports.default = ObjectGenerator;
//# sourceMappingURL=object-generator.js.map