"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectToIdiomaticEJSON = exports.fieldStringLen = void 0;
const bson_1 = require("bson");
function fieldStringLen(value) {
    const length = String(value).length;
    return length === 0 ? 1 : length;
}
exports.fieldStringLen = fieldStringLen;
function objectToIdiomaticEJSON(value, options = {}) {
    const serialized = bson_1.EJSON.serialize(value, {
        relaxed: false,
    });
    makeEJSONIdiomatic(serialized);
    return JSON.stringify(serialized, null, 'indent' in options ? options.indent : 2);
}
exports.objectToIdiomaticEJSON = objectToIdiomaticEJSON;
function makeEJSONIdiomatic(value) {
    if (!value || typeof value !== 'object')
        return;
    for (const key of Object.keys(value)) {
        const entry = value[key];
        if (typeof entry !== 'object' || entry === null) {
            continue;
        }
        if (entry.$numberInt) {
            value[key] = +entry.$numberInt;
            continue;
        }
        if (entry.$numberDouble) {
            value[key] = +entry.$numberDouble;
            continue;
        }
        makeEJSONIdiomatic(entry);
    }
}
//# sourceMappingURL=utils.js.map